mg_MapGenerator

	//==========================================================================
	var
		list/mg_doors

		//	Most procs in this library assume a rectangular map.
		//	If using different map shapes, you may wish to use mg_shape to keep track.
		const
			mg_RECTANGLE	= 1
		mg_shape			= mg_RECTANGLE

		//	These are the maximum map coordinates.
		//	The minimum for a valid rectangular map is considered to be 1.
		mg_X				= 0
		mg_Y				= 0

	//==========================================================================
	New(Bit=mg_NONE,SizeX,SizeY)
		if (!src.mg_dir2bit)
			src.mg_dir2bit = list(mg_NORTH,mg_SOUTH,mg_NONE,mg_EAST,mg_NORTHEAST,mg_SOUTHEAST,mg_NONE,mg_WEST,mg_NORTHWEST,mg_SOUTHWEST)
		src.mg_Reinitialize(Bit,SizeX,SizeY)

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	This returns the door index at the coordinates X and Y.
		//	Returns 0 if coordinates are out of the map's bounds
		mg_CoordinatesToIndex(X,Y)
			if ( !isnum(X) || (X<1) || (X>src.mg_X) )	return 0
			if ( !isnum(Y) || (Y<1) || (Y>src.mg_Y) )	return 0
			return max(0,(Y-1)*src.mg_X + X)

		//----------------------------------------------------------------------
		//	Returns the direction bit from the coordinates FromX,FromY to ToX,ToY.
		//	As implemented, this gets less accurate with distance as it assumes
		//	the cardinal directions are only on the X and Y axis.
		//	(It's only used internally on neighbors at the moment.)
		mg_GetDir(FromX,FromY,ToX,ToY)
			if ( !isnum(FromX) || (FromX<1) || (FromX>src.mg_doors.len) \
				|| !isnum(ToX) || (ToX<1) || (ToX>src.mg_doors.len) )
				return src.mg_NONE
			if ( !isnum(FromY) )
				FromY	= src.mg_IndexToCoordinates(FromX)
				FromX	= FromY[1]
				FromY	= FromY[2]
			if ( (FromX<1) || (FromX>src.mg_X) || (FromY<1) || (FromY>src.mg_Y) )
				return src.mg_NONE
			if ( !isnum(ToY) )
				ToY	= src.mg_IndexToCoordinates(ToX)
				ToX	= ToY[1]
				ToY	= ToY[2]
			if ( (ToX<1) || (ToX>src.mg_X) || (ToY<1) || (ToY>src.mg_Y) )
				return src.mg_NONE
			if (FromY<ToY)
				if (FromX<ToX)		return src.mg_NORTHEAST
				else if (FromX>ToX)	return src.mg_NORTHWEST
				return src.mg_NORTH
			if (FromY>ToY)
				if (FromX<ToX)		return src.mg_SOUTHEAST
				else if (FromX>ToX)	return src.mg_SOUTHWEST
				return src.mg_SOUTH
			if (FromX<ToX)		return src.mg_EAST
			else if (FromX>ToX)	return src.mg_WEST
			return src.mg_NONE

		//----------------------------------------------------------------------
		//	Returns the list of indices belonging to the cells connected to X,Y.
		//	If Y is not a number, it assumes X is an index.
		mg_GetConnections(X,Y)
			var/index
			if ( isnum(Y) )	index	= src.mg_CoordinatesToIndex(X,Y)
			else			index	= X
			if ((index<1) || (index>src.mg_doors.len))	return list()
			var/bits	= src.mg_doors[index]
			if (!bits)	return list()
			.				= list()
			if (bits&src.mg_NORTH)
				index		= src.mg_GetStep(src.mg_NORTH,X,Y)
				if (index)	.	+= index
			if (bits&src.mg_NORTHEAST)
				index		= src.mg_GetStep(src.mg_NORTHEAST,X,Y)
				if (index)	.	+= index
			if (bits&src.mg_EAST)
				index		= src.mg_GetStep(src.mg_EAST,X,Y)
				if (index)	.	+= index
			if (bits&src.mg_SOUTHEAST)
				index		= src.mg_GetStep(src.mg_SOUTHEAST,X,Y)
				if (index)	.	+= index
			if (bits&src.mg_SOUTH)
				index		= src.mg_GetStep(src.mg_SOUTH,X,Y)
				if (index)	.	+= index
			if (bits&src.mg_SOUTHWEST)
				index		= src.mg_GetStep(src.mg_SOUTHWEST,X,Y)
				if (index)	.	+= index
			if (bits&src.mg_WEST)
				index		= src.mg_GetStep(src.mg_WEST,X,Y)
				if (index)	.	+= index
			if (bits&src.mg_NORTHWEST)
				index		= src.mg_GetStep(src.mg_NORTHWEST,X,Y)
				if (index)	.	+= index

		//----------------------------------------------------------------------
		//	Returns the index of a step from the coordinates X,Y to	the
		//	direction of DirBit.  If Y is not a number, X is assumed to be
		//	an index.  DirBit is expected to be a single bit.
		//	Returns 0 on failure.
		mg_GetStep(DirBit,X,Y)
			if ( !DirBit || !(DirBit in src.mg_dir2bit) )		return 0
			if ( !isnum(X) || (X<1) || (X>src.mg_X*src.mg_Y) )	return 0
			X		= round(X)
			if ( !isnum(Y) )
				Y	= src.mg_IndexToCoordinates(X)
				X	= Y[1]
				Y	= Y[2]
				if ( (X<1) || (Y<1) )	return 0
			Y							= round(Y)
			if ( (X>src.mg_X) || (Y<1) || (Y>src.mg_Y) )		return 0
			if ( (DirBit&mg_NORTH) || (DirBit&mg_NORTHEAST) || (DirBit&mg_NORTHWEST) )		Y++
			else if ( (DirBit&mg_SOUTH) || (DirBit&mg_SOUTHEAST) || (DirBit&mg_SOUTHWEST) )	Y--
			if ( (DirBit&mg_EAST) || (DirBit&mg_NORTHEAST) || (DirBit&mg_SOUTHEAST) )			X++
			else if ( (DirBit&mg_WEST) || (DirBit&mg_NORTHWEST) || (DirBit&mg_SOUTHWEST) )	X--
			return src.mg_CoordinatesToIndex(X,Y)

		//----------------------------------------------------------------------
		//	Returns a list of turfs associated with the map's door bits with
		//	the assumption that X,Y,Z is the lower left corner.
		mg_GetTurfsToDoors(X=1,Y=1,Z=1)
			if ( !isnum(X) || !isnum(Y) || !isnum(Z) )	return list()
			X											= round(X)
			Y											= round(Y)
			Z											= round(Z)
			if ( (Z<1) || (Z>world.maxz) )				return list()
			if ( (Y<1) || (Y+src.mg_Y-1 > world.maxy) )	return list()
			if ( (X<1) || (X+src.mg_X-1 > world.maxx) )	return list()
			var
				turfsToDoors	= list()
				index
				coordinates
			for(index = 1 to src.mg_doors.len)
				coordinates		= src.mg_IndexToCoordinates(index)
				turfsToDoors[locate(X+coordinates[1]-1,Y+coordinates[2]-1,Z)]	= src.mg_doors[index]
			return turfsToDoors

		//----------------------------------------------------------------------
		//	Converts Index into a list of coordinates.
		//	Returns list(0,0) if no such coordinates exist.
		mg_IndexToCoordinates(Index)
			if ( !isnum(Index) || (Index<1) || (Index>src.mg_doors.len) )
				return list(0,0)
			var/Y				= Index/src.mg_X
			if (Y != round(Y))	Y	= round(Y) + 1
			return list(Index - (Y-1)*src.mg_X,Y)

		//----------------------------------------------------------------------
		//	Returns whether or not the cell at X1,Y1 is connected to X2,Y2.
		//	If eithe rY is null, it assumes the corresponding X is an index.
		mg_IsConnected(X1,Y1,X2,Y2)
			if ( !isnum(X1) || !isnum(X2) )	return FALSE
			if ( isnum(Y2) )	X2	= src.mg_CoordinatesToIndex(X2,Y2)
			return (X2 in src.mg_GetConnections(X1,Y1))

		//----------------------------------------------------------------------
		//	Sets all doors to Bit.  If X and Y are numbers, the map is resized
		//	with the assumption that it is a rectangle with X*Y cells.
		mg_Reinitialize(Bit=mg_NONE,X,Y)
			if (!src.mg_doors)				src.mg_doors	= list()
			if ( isnum(X) && isnum(Y) )
				X							= round(X)
				Y							= round(Y)
				if ( (X<1) || (Y<1) )		return FALSE
				src.mg_doors.len			= X*Y
				src.mg_X					= X
				src.mg_Y					= Y
			for(var/index = 1 to src.mg_doors.len)	src.mg_doors[index]	= Bit
