mg_MapGenerator

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Sets the generator's doors to form a maze using Prim's maze algorithm.
		//	Prim's algorithm guarantees the use of all available cells.
		//	Assumes a rectangle.  If NESW, diagonals are ignored.
		//	If Seed or the starting coordinates are invalid, valid ones will be chosen.
		//	Returns list(Seed,Starting Index).
		mg_PrimMazeRectangle(Seed,StartX,StartY,NESW=TRUE)
			//	Clear all doors.
			src.mg_Reinitialize(src.mg_NONE)
			//	Find the actual seed.
			if ( !isnum(Seed) )	Seed	= world.time
			rand_seed(Seed)
			//	Find the actual starting index.
			var/indexIn			= src.mg_CoordinatesToIndex(StartX,StartY)
			if (!indexIn)		indexIn	= rand(1,src.mg_doors.len)
			//	Make sure the actual values are returned.
			.	= list(Seed,indexIn)
			//	Mark the starting node as "in"
			var/list/nodesIn	= list(indexIn)
			//	Find how many neighbor directions per node.
			var
				list/opposites	= src.mg_GetBitsAndOpposites()
				list/dirs		= opposites[1]
			opposites			= opposites[2]
			if (NESW)
				dirs.Cut(5,0)
				opposites.Cut(5,0)
			//	Find "frontier" neighbors of the starting node.
			var
				list/nodesFrontier	= list()
				indexFrontier
				D
			for(D in dirs)
				indexFrontier		= src.mg_GetStep(D,indexIn)
				if (indexFrontier)	nodesFrontier	+= indexFrontier
			var/list/neighborsIn	= list()
			while(nodesFrontier.len)
				//	Pick a "frontier" node.
				indexFrontier	= nodesFrontier[rand(1,nodesFrontier.len)]
				nodesFrontier	-= indexFrontier
				//	Find the node's "in" neighbors.
				neighborsIn.Cut()
				for(D in dirs)
					indexIn		= src.mg_GetStep(D,indexFrontier)
					if (indexIn)
						if ( nodesIn.Find(indexIn) )
							neighborsIn		+= indexIn
						//	Collect "out" neighbors for the frontier.
						else if ( !nodesFrontier.Find(indexIn) )
							nodesFrontier	+= indexIn
				//	Pick a random "in" neighbor.
				indexIn	= neighborsIn[rand(1,neighborsIn.len)]
				//	Link the "frontier" node to the "in" node and vice versa.
				D	= src.mg_GetDir(indexFrontier,null,indexIn,null)
				src.mg_doors[indexFrontier]	|= D
				src.mg_doors[indexIn]		|= opposites[dirs.Find(D)]
				//	Mark the "frontier" node as "in".
				nodesIn	+= indexFrontier
