/*

Map Generation library by ACWraith


////////////////////////////////////////////////////////////////////////////////
LICENSE:
	You may use this library in both commercial and non-commercial projects.
	Credit would be appreciated, but it is optional.
////////////////////////////////////////////////////////////////////////////////


DESCRIPTION:

This library provides a datum for abstract map generation with:
	*	No need for atoms until the map is applied.
	*	No need to spend entire cells on walls.
			Each cell knows whether it's connected to its neighbors or not.
	*	Procs for pathfinding without applying the map first.

At the time of this writing, the only actual generation provided is for a
rectangular maze using Prim's algorithm.


////////////////////////////////////////////////////////////////////////////////

mg_MapGenerator


	New(Bit=mg_NONE,SizeX,SizeY)
		New pretty much just calls mg_Reinitialize()

	mg_Reinitialize(Bit=mg_NONE,X,Y)
		Sets all doors to Bit.  If X and Y are numbers, the map is resized
		with the assumption that it is a rectangle with X*Y cells.


	The map is implemented as an array.  If not using turfs as cells, you will
	likely wish to reference it directly.

		list/mg_doors


	By default, the array is assumed to be a rectangle with the following dimensions:

		mg_X	= 0
		mg_Y	= 0


	Developers can convert between the map coordinates and the array indices.

		mg_CoordinatesToIndex(X,Y)
			This returns the door index at the coordinates X and Y.
			Returns 0 if coordinates are out of the map's bounds

		mg_IndexToCoordinates(Index)
			Converts Index into a list of coordinates.
			Returns list(0,0) if no such coordinates exist.


	Each cell in the mg_doors array is a number with bits corresponding to the
	connected directions.  A lack of a bit means the lack of a path.

		The directional bits are defined the same as those in
		Lummox JR's Icon Cutter:
			mg_NONE
			mg_NORTH
			mg_NORTHEAST
			mg_EAST
			mg_SOUTHEAST
			mg_SOUTH
			mg_SOUTHWEST
			mg_WEST
			mg_NORTHWEST

		mg_BitToDir(Bit)
			Converts the mg bit Bit into its BYOND direction equivalent.

		mg_DirToBit(Dir)
			Converts the BYOND direction Dir into its mg bit equivalent.

		mg_GetBitsAndOpposites()
			Returns list(list of directional bits, list of opposing directional bits).


	The map can be navigated without being applied to atoms.

		mg_GetDir(FromX,FromY,ToX,ToY)
			Returns the direction bit from the coordinates FromX,FromY to ToX,ToY.
			As implemented, this gets less accurate with distance as it assumes
			the cardinal directions are only on the X and Y axis.
			(It's only used internally on neighbors at the moment.)

		mg_GetStep(DirBit,X,Y)
			Returns the index of a step from the coordinates X,Y to	the
			direction of DirBit.  If Y is not a number, X is assumed to be
			an index.  DirBit is expected to be a single bit.
			Returns 0 on failure.

			NOTE: 	This pays no attention to whether or not there is a
					connection between the cell given and the cell received.


	Similarly, connections can be tested for pathfinding.

		mg_GetConnections(X,Y)
			Returns the list of indices belonging to the cells connected to X,Y.
			If Y is not a number, it assumes X is an index.

		mg_IsConnected(X1,Y1,X2,Y2)
			Returns whether or not the cell at X1,Y1 is connected to X2,Y2.
			If eithe rY is null, it assumes the corresponding X is an index.


	At the moment, there is only one map generating algorithm provided by default.

		mg_PrimMazeRectangle(Seed,StartX,StartY,NESW=TRUE)
			Sets the generator's doors to form a maze using Prim's maze algorithm.
			Prim's algorithm guarantees the use of all available cells.
			Assumes a rectangle.  If NESW, diagonals are ignored.
			If Seed or the starting coordinates are invalid, valid ones will be chosen.
			Returns list(Seed,Starting Index).


	The following is provided to ease the use of turfs as map cells.

		mg_GetTurfsToDoors(X=1,Y=1,Z=1)
			Returns a list of turfs associated with the map's door bits with
			the assumption that X,Y,Z is the lower left corner.

*/